@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2013-2022, The PurpleI2P Project
+* Copyright (c) 2013-2024, The PurpleI2P Project
 *
 * This file is part of Purple i2pd project and licensed under BSD3
 *
@@ -12,8 +12,10 @@
 #include <inttypes.h>
 #include <string>
 #include <memory>
+#include <mutex>
 #include <thread>
 #include <map>
+#include <unordered_map>
 #include <boost/asio.hpp>
 #include "util.h"
 #include "Destination.h"
@@ -58,6 +60,7 @@ namespace client
 		eI2CPMessageStatusAccepted = 1,
 		eI2CPMessageStatusGuaranteedSuccess = 4,
 		eI2CPMessageStatusGuaranteedFailure = 5,
+		eI2CPMessageStatusNoLocalTunnels = 16,
 		eI2CPMessageStatusNoLeaseSet = 21
 	};
 
@@ -79,7 +82,8 @@ namespace client
 		public:
 
 			I2CPDestination (boost::asio::io_service& service, std::shared_ptr<I2CPSession> owner,
-				std::shared_ptr<const i2p::data::IdentityEx> identity, bool isPublic, const std::map<std::string, std::string>& params);
+				std::shared_ptr<const i2p::data::IdentityEx> identity, bool isPublic, bool isSameThread, 
+			    const std::map<std::string, std::string>& params);
 			~I2CPDestination () {};
 
 			void Stop ();
@@ -90,7 +94,8 @@ namespace client
 			void LeaseSetCreated (const uint8_t * buf, size_t len); // called from I2CPSession
 			void LeaseSet2Created (uint8_t storeType, const uint8_t * buf, size_t len); // called from I2CPSession
 			void SendMsgTo (const uint8_t * payload, size_t len, const i2p::data::IdentHash& ident, uint32_t nonce); // called from I2CPSession
-
+			bool SendMsg (const uint8_t * payload, size_t len, std::shared_ptr<i2p::garlic::GarlicRoutingSession> remoteSession, uint32_t nonce);
+			
 			// implements LocalDestination
 			bool Decrypt (const uint8_t * encrypted, uint8_t * data, i2p::data::CryptoKeyType preferredCrypto) const;
 			bool SupportsEncryptionType (i2p::data::CryptoKeyType keyType) const;
@@ -99,6 +104,7 @@ namespace client
 
 		protected:
 
+			void CleanupDestination ();
 			// I2CP
 			void HandleDataMessage (const uint8_t * buf, size_t len);
 			void CreateNewLeaseSet (const std::vector<std::shared_ptr<i2p::tunnel::InboundTunnel> >& tunnels);
@@ -108,7 +114,9 @@ namespace client
 			std::shared_ptr<I2CPDestination> GetSharedFromThis ()
 			{ return std::static_pointer_cast<I2CPDestination>(shared_from_this ()); }
 			bool SendMsg (std::shared_ptr<I2NPMessage> msg, std::shared_ptr<const i2p::data::LeaseSet> remote);
-
+			bool SendMsg (std::shared_ptr<I2NPMessage> garlic, 
+				std::shared_ptr<i2p::tunnel::OutboundTunnel> outboundTunnel, std::shared_ptr<const i2p::data::Lease> remoteLease);
+			
 			void PostCreateNewLeaseSet (std::vector<std::shared_ptr<i2p::tunnel::InboundTunnel> > tunnels);
 
 		private:
@@ -120,7 +128,7 @@ namespace client
 			std::shared_ptr<i2p::crypto::ECIESX25519AEADRatchetDecryptor> m_ECIESx25519Decryptor;
 			uint8_t m_ECIESx25519PrivateKey[32];
 			uint64_t m_LeaseSetExpirationTime;
-			bool m_IsCreatingLeaseSet;
+			bool m_IsCreatingLeaseSet, m_IsSameThread;
 			boost::asio::deadline_timer m_LeaseSetCreationTimer;
 			i2p::util::MemoryPoolMt<I2NPMessageBuffer<I2NP_MAX_MESSAGE_SIZE> > m_I2NPMsgsPool;
 	};
@@ -155,6 +163,8 @@ namespace client
 			void SendI2CPMessage (uint8_t type, const uint8_t * payload, size_t len);
 			void SendMessagePayloadMessage (const uint8_t * payload, size_t len);
 			void SendMessageStatusMessage (uint32_t nonce, I2CPMessageStatus status);
+			void AddRoutingSession (const i2p::data::IdentHash& signingKey, std::shared_ptr<i2p::garlic::GarlicRoutingSession> remoteSession);
+			void CleanupRoutingSessions ();
 
 			// message handlers
 			void GetDateMessageHandler (const uint8_t * buf, size_t len);
@@ -186,7 +196,7 @@ namespace client
 			void ExtractMapping (const uint8_t * buf, size_t len, std::map<std::string, std::string>& mapping);
 			void SendSessionStatusMessage (I2CPSessionStatus status);
 			void SendHostReplyMessage (uint32_t requestID, std::shared_ptr<const i2p::data::IdentityEx> identity);
-
+			
 		private:
 
 			I2CPServer& m_Owner;
@@ -195,6 +205,8 @@ namespace client
 			size_t m_PayloadLen;
 
 			std::shared_ptr<I2CPDestination> m_Destination;
+			std::mutex m_RoutingSessionsMutex;
+			std::unordered_map<i2p::data::IdentHash, std::shared_ptr<i2p::garlic::GarlicRoutingSession> > m_RoutingSessions; // signing key->session
 			uint16_t m_SessionID;
 			uint32_t m_MessageID;
 			bool m_IsSendAccepted;
@@ -210,7 +222,7 @@ namespace client
 	{
 		public:
 
-			I2CPServer (const std::string& interface, int port, bool isSingleThread);
+			I2CPServer (const std::string& interface, uint16_t port, bool isSingleThread);
 			~I2CPServer ();
 
 			void Start ();
@@ -224,8 +236,6 @@ namespace client
 
 		private:
 
-			void Run ();
-
 			void Accept ();
 
 			void HandleAccept(const boost::system::error_code& ecode, std::shared_ptr<boost::asio::ip::tcp::socket> socket);
